module Bamboozled
  module API
    class Employee < Base

      def all(fields = nil)
        response = request(:get, "employees/directory")

        if fields.nil? || fields == :default
          Array(response['employees'])
        else
          employees = []
          response['employees'].map{|e| e['id']}.each do |id|
            employees << find(id, fields)
          end
          employees
        end
      end

      def find(employee_id, fields = nil)
        fields = FieldCollection.wrap(fields).to_csv

        request(:get, "employees/#{employee_id}?fields=#{fields}&onlyCurrent=false")
      end

      def last_changed(date = "2011-06-05T00:00:00+00:00", type = nil)
        query = Hash.new
        query[:since] = date.respond_to?(:iso8601) ? date.iso8601 : date
        query[:type] = type unless type.nil?

        response = request(:get, "employees/changed", query: query)
        response["employees"]
      end

      # Tabular data
      [:job_info, :employment_status, :compensation, :dependents, :contacts].each do |action|
        define_method(action.to_s) do |argument_id|
          request(:get, "employees/#{argument_id}/tables/#{action.to_s.gsub(/_(.)/) {|e| $1.upcase}}")
        end
      end

      def add_table_row(employee_id, table_name, table_row_data)
        details = generate_xml(table_row_data)
        options = {body: details}
        request(:post, "employees/#{employee_id}/tables/#{table_name}", options)
      end

      def update_table_row(employee_id, table_name, row_id, table_row_data)
        details = generate_xml(table_row_data)
        options = {body: details}
        request(:post, "employees/#{employee_id}/tables/#{table_name}/#{row_id}", options)
      end

      def table_data(employee_id, table_name)
        request(:get, "employees/#{employee_id}/tables/#{table_name}")
      end

      def time_off_estimate(employee_id, end_date)
        end_date = end_date.strftime("%F") unless end_date.is_a?(String)
        request(:get, "employees/#{employee_id}/time_off/calculator?end=#{end_date}")
      end

      def time_off_balance_adjustment(employee_id, params)
        allowed_parameters = %i[date timeOffTypeId amount note]
        params = params.keep_if { |k, _| allowed_parameters.include? k }

        options = {
          body: params.to_json,
          headers: { "Content-Type" => "application/json" }
        }

        request(:put, "employees/#{employee_id}/time_off/balance_adjustment", options)
      end

      def time_off_policies(employee_id)
        request(:get, "employees/#{employee_id}/time_off/policies", { api_version: 'v1_1' })
      end

      def add_time_off_policies(employee_id, policies)
        allowed_keys = %i[timeOffPolicyId accrualStartDate]
        policies = policies.map { |policy| policy.keep_if { |k, _| allowed_keys.include? k } }

        options = {
          body: policies.to_json
        }

        request(:put, "employees/#{employee_id}/time_off/policies", options)
      end

      def photo_binary(employee_id)
        request(:get, "employees/#{employee_id}/photo/small")
      end

      def photo_url(employee)
        if (Float(employee) rescue false)
          e = find(employee, ['workEmail', 'homeEmail'])
          employee = e['workEmail'].nil? ? e['homeEmail'] : e['workEmail']
        end

        digest = Digest::MD5.new
        digest.update(employee.strip.downcase)
        "http://#{@subdomain}.bamboohr.com/employees/photos/?h=#{digest}"
      end

      def add(employee_details)
        details = generate_xml(employee_details)
        options = {body: details}

        request(:post, "employees/", options)
      end

      def update(bamboo_id, employee_details)
        details = generate_xml(employee_details)
        options = { body: details }

        request(:post, "employees/#{bamboo_id}", options)
      end

      def files(employee_id)
        request(:get, "employees/#{employee_id}/files/view/")
      end

      def add_file(employee_id, file_details)
        post_file("employees/#{employee_id}/files", file_details)
      end

      private

      def generate_xml(employee_details)
        "".tap do |xml|
          xml << "<employee>"
          employee_details.each do |k, v|
            if v.is_a?(Hash)
              value = Integer(v[:value], exception: false) ? v[:value] : v[:value].encode(xml: :text)
              xml <<  "<field id='#{k}' currency='#{v[:currency]}'>#{value}</field>"
            else
              value = Integer(v, exception: false) ? v : v.encode(xml: :text)
              xml << "<field id='#{k}'>#{value}</field>"
            end
          end
          xml << "</employee>"
        end
      end
    end
  end
end
